@@ -6,12 +6,13 @@
 use std::{
     io::{self, BufReader},
     path::PathBuf,
-    sync::Arc,
+    sync::{Arc, Mutex},
 };
 
 use crate::{
     llconn,
     msgs::{
+        request::InvalidRequestError,
         response::{ResponseKind, RpcError, ValidatedResponse},
         AnyRequestId, ObjectId,
     },
@@ -26,13 +27,21 @@ pub use connimpl::RpcConn;
 
 /// A handle to an open request.
 ///
-/// These handles are crated with [`RpcConn::execute_with_handle`].
+/// These handles are created with [`RpcConn::execute_with_handle`].
 #[derive(educe::Educe)]
 #[educe(Debug)]
 pub struct RequestHandle {
     /// The underlying `Receiver` that we'll use to get updates for this request
+    ///
+    /// It's wrapped in a `Mutex` to prevent concurrent calls to `Receiver::wait_on_message_for`.
+    //
+    // NOTE: As an alternative to using a Mutex here, we _could_ remove
+    // the restriction from `wait_on_message_for` that says that only one thread
+    // may be waiting on a given request ID at once.  But that would introduce
+    // complexity to the implementation,
+    // and it's not clear that the benefit would be worth it.
     #[educe(Debug(ignore))]
-    conn: Arc<connimpl::Receiver>,
+    conn: Mutex<Arc<connimpl::Receiver>>,
     /// The ID of this request.
     id: AnyRequestId,
 }
@@ -67,7 +76,7 @@ pub struct SuccessResponse(Utf8CString);
 // Invariant: it is valid JSON and contains no NUL bytes or newlines.
 // TODO RPC: check that the newline invariant is enforced in constructors.
 // TODO RPC consider changing this to CString.
-#[derive(Clone, Debug, derive_more::AsRef)]
+#[derive(Clone, Debug, derive_more::AsRef, derive_more::Into)]
 #[as_ref(forward)]
 pub struct UpdateResponse(Utf8CString);
 
@@ -84,7 +93,7 @@ pub struct UpdateResponse(Utf8CString);
 // Otherwise the `decode` method may panic.
 //
 // TODO RPC: check that the newline invariant is enforced in constructors.
-#[derive(Clone, Debug, derive_more::AsRef)]
+#[derive(Clone, Debug, derive_more::AsRef, derive_more::Into)]
 #[as_ref(forward)]
 // TODO: If we keep this, it should implement Error.
 pub struct ErrorResponse(Utf8CString);
@@ -210,6 +219,16 @@ impl AnyResponse {
             ResponseKind::Update => AnyResponse::Update(UpdateResponse(v.msg)),
         }
     }
+
+    /// Consume this `AnyResponse`, and return its internal string.
+    #[cfg(feature = "ffi")]
+    pub(crate) fn into_string(self) -> Utf8CString {
+        match self {
+            AnyResponse::Success(m) => m.into(),
+            AnyResponse::Error(m) => m.into(),
+            AnyResponse::Update(m) => m.into(),
+        }
+    }
 }
 
 impl RpcConn {
@@ -254,7 +273,7 @@ impl RpcConn {
     where
         F: FnMut(UpdateResponse) + Send + Sync,
     {
-        let mut hnd = self.execute_with_handle(cmd)?;
+        let hnd = self.execute_with_handle(cmd)?;
         loop {
             match hnd.wait_with_updates()? {
                 AnyResponse::Success(s) => return Ok(Ok(s)),
@@ -279,7 +298,7 @@ impl RequestHandle {
     /// Note that this function will return `Err(.)` only if sending the command or getting a
     /// response failed.  If the command was sent successfully, and Arti reported an error in response,
     /// this function returns `Ok(Err(.))`.
-    pub fn wait(mut self) -> Result<FinalResponse, ProtoError> {
+    pub fn wait(self) -> Result<FinalResponse, ProtoError> {
         loop {
             match self.wait_with_updates()? {
                 AnyResponse::Success(s) => return Ok(Ok(s)),
@@ -294,11 +313,15 @@ impl RequestHandle {
     /// response failed.  If the command was sent successfully, and Arti reported an error in response,
     /// this function returns `Ok(AnyResponse::Error(.))`.
     ///
+    /// You may call this method on the same `RequestHandle` from multiple threads.
+    /// If you do so, those calls will receive responses (or errors) in an unspecified order.
+    ///
     /// If this function returns Success or Error, then you shouldn't call it again.
     /// All future calls to this function will fail with `CmdError::RequestCancelled`.
     /// (TODO RPC: Maybe rename that error.)
-    pub fn wait_with_updates(&mut self) -> Result<AnyResponse, ProtoError> {
-        let validated = self.conn.wait_on_message_for(&self.id)?;
+    pub fn wait_with_updates(&self) -> Result<AnyResponse, ProtoError> {
+        let conn = self.conn.lock().expect("Poisoned lock");
+        let validated = conn.wait_on_message_for(&self.id)?;
 
         Ok(AnyResponse::from_validated(validated))
     }
@@ -315,10 +338,10 @@ impl RequestHandle {
 pub enum ShutdownError {
     /// Io error occurred while reading.
     #[error("Unable to read response: {0}")]
-    Read(Arc<io::Error>),
+    Read(#[source] Arc<io::Error>),
     /// Io error occurred while writing.
     #[error("Unable to write request: {0}")]
-    Write(Arc<io::Error>),
+    Write(#[source] Arc<io::Error>),
     /// Something was wrong with Arti's responses; this is a protocol violation.
     #[error("Arti sent a message that didn't conform to the RPC protocol: {0}")]
     ProtocolViolated(String),
@@ -354,7 +377,7 @@ pub enum ProtoError {
 
     /// There was a problem in the request we tried to send.
     #[error("Invalid request: {0}")]
-    InvalidRequest(Arc<serde_json::Error>),
+    InvalidRequest(#[from] InvalidRequestError),
 
     /// We tried to send a request with an ID that was already pending.
     #[error("Request ID already in use.")]
@@ -377,7 +400,7 @@ pub enum ProtoError {
     ///
     /// (This should be impossible.)
     #[error("Internal error while encoding request: {0}")]
-    CouldNotEncode(Arc<serde_json::Error>),
+    CouldNotEncode(#[source] Arc<serde_json::Error>),
 }
 
 /// An error while trying to connect to the Arti process.
@@ -390,13 +413,13 @@ pub enum ConnectError {
     SchemeNotSupported,
     /// IO error while connecting to Arti.
     #[error("Unable to make a connection: {0}")]
-    CannotConnect(Arc<std::io::Error>),
+    CannotConnect(#[source] Arc<std::io::Error>),
     /// One of our authentication messages was rejected.
     #[error("Arti rejected our authentication: {0:?}")]
     AuthenticationRejected(ErrorResponse),
     /// We couldn't decode one of the responses we got.
     #[error("Message not in expected format: {0:?}")]
-    BadMessage(Arc<serde_json::Error>),
+    BadMessage(#[source] Arc<serde_json::Error>),
     /// A protocol error occurred during negotiations.
     #[error("Error while negotiating with Arti: {0}")]
     ProtoError(#[from] ProtoError),
@@ -434,7 +457,7 @@ mod test {
     use rand::{seq::SliceRandom as _, Rng as _, SeedableRng as _};
     use tor_basic_utils::{test_rng::testing_rng, RngExt as _};
 
-    use crate::msgs::request::{JsonMap, Request};
+    use crate::msgs::request::{JsonMap, Request, ValidatedRequest};
 
     use super::*;
 
@@ -469,9 +492,9 @@ mod test {
             let mut sock = BufReader::new(sock);
             let mut s = String::new();
             let _len = sock.read_line(&mut s).unwrap();
-            let request: Request<JsonMap> = serde_json::from_str(&s).unwrap();
+            let request = ValidatedRequest::from_string_strict(s.as_ref()).unwrap();
             let response = serde_json::json!({
-                "id": request.id.clone(),
+                "id": request.id().clone(),
                 "result": { "xyz" : 3 }
             });
             write_val(sock.get_mut(), &response);
@@ -665,10 +688,19 @@ mod test {
                 });
                 let req = serde_json::to_string(&req).unwrap();
                 let outcome = conn.execute(&req);
+                if !matches!(
+                    &outcome,
+                    Err(ProtoError::Shutdown(ShutdownError::Write(_)))
+                        | Err(ProtoError::Shutdown(ShutdownError::Read(_))),
+                ) {
+                    dbg!(&outcome);
+                }
+
                 assert!(matches!(
                     outcome,
                     Err(ProtoError::Shutdown(ShutdownError::Write(_)))
                         | Err(ProtoError::Shutdown(ShutdownError::Read(_)))
+                        | Err(ProtoError::Shutdown(ShutdownError::ConnectionClosed))
                 ));
             });
             user_threads.push(th);
