@@ -212,31 +212,31 @@ fn default_rpc_path() -> Option<CfgPath> {
 /// stability here.
 #[derive(Debug, Builder, Clone, Eq, PartialEq)]
 #[builder(derive(Serialize, Deserialize, Debug))]
-#[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(build_fn(private, name = "build_unvalidated", error = "ConfigBuildError"))]
 pub struct ArtiConfig {
     /// Configuration for application behavior.
-    #[builder(sub_builder)]
+    #[builder(sub_builder(fn_name = "build"))]
     #[builder_field_attr(serde(default))]
     application: ApplicationConfig,
 
     /// Configuration for proxy listeners
-    #[builder(sub_builder)]
+    #[builder(sub_builder(fn_name = "build"))]
     #[builder_field_attr(serde(default))]
     proxy: ProxyConfig,
 
     /// Logging configuration
-    #[builder(sub_builder)]
+    #[builder(sub_builder(fn_name = "build"))]
     #[builder_field_attr(serde(default))]
     logging: LoggingConfig,
 
     /// Configuration for RPC subsystem
     #[cfg(feature = "rpc")]
-    #[builder(sub_builder)]
+    #[builder(sub_builder(fn_name = "build"))]
     #[builder_field_attr(serde(default))]
     rpc: RpcConfig,
 
     /// Information on system resources used by Arti.
-    #[builder(sub_builder)]
+    #[builder(sub_builder(fn_name = "build"))]
     #[builder_field_attr(serde(default))]
     pub(crate) system: SystemConfig,
 
@@ -248,13 +248,29 @@ pub struct ArtiConfig {
     /// The purpose of this stub type is to give an error if somebody tries to
     /// configure onion services when the `onion-service-service` feature is
     /// disabled.
-    #[builder(sub_builder, setter(custom))]
+    #[builder(sub_builder(fn_name = "build"), setter(custom))]
     #[builder_field_attr(serde(default))]
     pub(crate) onion_services: OnionServiceProxyConfigMap,
 }
 
 impl_standard_builder! { ArtiConfig }
 
+impl ArtiConfigBuilder {
+    /// Build the [`ArtiConfig`].
+    pub fn build(&self) -> Result<ArtiConfig, ConfigBuildError> {
+        let mut config = self.build_unvalidated()?;
+        #[cfg(feature = "onion-service-service")]
+        for svc in config.onion_services.values_mut() {
+            // Pass the application-level watch_configuration to each restricted discovery config.
+            *svc.svc_cfg
+                .restricted_discovery_mut()
+                .watch_configuration_mut() = config.application.watch_configuration;
+        }
+
+        Ok(config)
+    }
+}
+
 impl tor_config::load::TopLevel for ArtiConfig {
     type Builder = ArtiConfigBuilder;
     const DEPRECATED_KEYS: &'static [&'static str] = &["proxy.socks_port", "proxy.dns_port"];
@@ -325,6 +341,15 @@ mod test {
     use std::time::Duration;
     use tor_config::load::{ConfigResolveError, ResolutionResults};
 
+    #[cfg(feature = "restricted-discovery")]
+    use {
+        arti_client::HsClientDescEncKey,
+        std::str::FromStr as _,
+        tor_hsservice::config::restricted_discovery::{
+            DirectoryKeyProviderBuilder, HsClientNickname,
+        },
+    };
+
     use super::*;
 
     //---------- tests that rely on the provided example config file ----------
@@ -1092,12 +1117,13 @@ example config file {which:?}, uncommented={uncommented:?}
         file.strip_prefix("#    ");
 
         let result = file.resolve::<(TorClientConfig, ArtiConfig)>();
+        let cfg_got = result.unwrap();
+
         #[cfg(feature = "pt-client")]
         {
             use arti_client::config::{pt::TransportConfig, BridgesConfig};
             use tor_config::CfgPath;
 
-            let cfg_got = result.unwrap();
             let bridges_got: &BridgesConfig = cfg_got.0.as_ref();
 
             // Build the expected configuration.
@@ -1125,21 +1151,20 @@ example config file {which:?}, uncommented={uncommented:?}
     #[test]
     fn onion_services() {
         // Here we require that the onion services configuration is between a
-        // line labeled with "#     [onion_service" a line that looks like a
-        // section opener, and that each line of _real_ configuration in that
-        // section begins with "#    ".
+        // line labeled with `#     [onion_service."allium-cepa"]` and
+        // a line that contains the start of the [vanguards] section,
+        // and that each line of _real_ configuration in that section begins with "#    ".
         let mut file = ExampleSectionLines::from_string(ARTI_EXAMPLE_CONFIG);
-        file.narrow((r"^#    \[onion_service", true), (r"^#? *\]", true));
+        file.narrow(
+            (r#"^#    \[onion_services."allium\-cepa"\]"#, true),
+            (r#"^\[vanguards\]"#, true),
+        );
         file.lines.retain(|line| line.starts_with("#    "));
         file.strip_prefix("#    ");
 
         let result = file.resolve::<(TorClientConfig, ArtiConfig)>();
         #[cfg(feature = "onion-service-service")]
         {
-            let cfg = result.unwrap();
-            let services = cfg.1.onion_services;
-            assert_eq!(services.len(), 1);
-            let svc = services.values().next().unwrap();
             let svc_expected = {
                 use tor_hsrproxy::config::*;
                 let mut b = OnionServiceProxyConfigBuilder::default();
@@ -1172,9 +1197,53 @@ example config file {which:?}, uncommented={uncommented:?}
                     ProxyPattern::all_ports(),
                     ProxyAction::DestroyCircuit,
                 ));
+
+                #[cfg(feature = "restricted-discovery")]
+                {
+                    const ALICE_KEY: &str =
+                        "descriptor:x25519:PU63REQUH4PP464E2Y7AVQ35HBB5DXDH5XEUVUNP3KCPNOXZGIBA";
+                    const BOB_KEY: &str =
+                        "descriptor:x25519:b5zqgtpermmuda6vc63lhjuf5ihpokjmuk26ly2xksf7vg52aesq";
+                    for (nickname, key) in [("alice", ALICE_KEY), ("bob", BOB_KEY)] {
+                        b.service()
+                            .restricted_discovery()
+                            .enabled(true)
+                            .static_keys()
+                            .access()
+                            .push((
+                                HsClientNickname::from_str(nickname).unwrap(),
+                                HsClientDescEncKey::from_str(key).unwrap(),
+                            ));
+                    }
+                    let mut dir = DirectoryKeyProviderBuilder::default();
+                    dir.path(CfgPath::new(
+                        "/var/lib/tor/hidden_service/authorized_clients".to_string(),
+                    ));
+
+                    b.service()
+                        .restricted_discovery()
+                        .key_dirs()
+                        .access()
+                        .push(dir);
+                }
+
                 b.build().unwrap()
             };
-            assert_eq!(svc, &svc_expected);
+
+            cfg_if::cfg_if! {
+                if #[cfg(feature = "restricted-discovery")] {
+                    let cfg = result.unwrap();
+                    let services = cfg.1.onion_services;
+                    assert_eq!(services.len(), 1);
+                    let svc = services.values().next().unwrap();
+                    assert_eq!(svc, &svc_expected);
+                } else {
+                    expect_err_contains(
+                        result.unwrap_err(),
+                        "restricted_discovery.enabled=true, but restricted-discovery feature not enabled"
+                    );
+                }
+            }
         }
         #[cfg(not(feature = "onion-service-service"))]
         {
