@@ -105,6 +105,7 @@ pub use logging::{LoggingConfig, LoggingConfigBuilder};
 use arti_client::config::default_config_files;
 use arti_client::{TorClient, TorClientConfig};
 use safelog::with_safe_logging_suppressed;
+use tor_config::mistrust::BuilderExt as _;
 use tor_config::{ConfigurationSources, Listen};
 use tor_rtcompat::{BlockOn, Runtime};
 
@@ -245,15 +246,14 @@ async fn run<R: Runtime>(
     )?;
 
     #[cfg(all(feature = "rpc", feature = "tokio"))]
-    let rpc_mgr = {
+    let rpc_data = {
         // TODO RPC This code doesn't really belong here; it's just an example.
         if let Some(listen_path) = rpc_path {
+            let (rpc_state, rpc_state_sender) = rpc::RpcVisibleArtiState::new();
             // TODO Conceivably this listener belongs on a renamed "proxy" list.
-            Some(rpc::launch_rpc_listener(
-                &runtime,
-                listen_path,
-                client.clone(),
-            )?)
+            let rpc_mgr =
+                rpc::launch_rpc_listener(&runtime, listen_path, client.clone(), rpc_state)?;
+            Some((rpc_mgr, rpc_state_sender))
         } else {
             None
         }
@@ -269,7 +269,7 @@ async fn run<R: Runtime>(
                 client,
                 socks_listen,
                 #[cfg(all(feature = "rpc", feature = "tokio"))]
-                rpc_mgr,
+                rpc_data,
             )
             .await;
             (res, "SOCKS")
@@ -512,8 +512,7 @@ where
             fs_mistrust::Mistrust::new_dangerously_trust_everyone()
         } else {
             fs_mistrust::MistrustBuilder::default()
-                .controlled_by_env_var(arti_client::config::FS_PERMISSIONS_CHECKS_DISABLE_VAR)
-                .build()
+                .build_for_arti()
                 .expect("Could not construct default fs-mistrust")
         };
 
