@@ -1,10 +1,18 @@
 #![doc = include_str!("../README.md")]
 
+/// The value to set the `COLUMNS` environment variable to.
+///
+/// Set to a large value to suppress line wrapping.
+///
+/// See <https://github.com/assert-rs/snapbox/issues/361>
+const COLUMNS: usize = 1000;
+
 #[test]
 fn cli_tests() {
     let t = trycmd::TestCases::new();
     let dir = tempfile::TempDir::new().unwrap();
-    t.env("HOME", dir.path().to_str().unwrap());
+    t.env("HOME", dir.path().to_str().unwrap())
+        .env("COLUMNS", COLUMNS.to_string());
 
     cfg_if::cfg_if! {
         if #[cfg(feature = "onion-service-service")] {
@@ -19,12 +27,8 @@ fn cli_tests() {
 
     cfg_if::cfg_if! {
         if #[cfg(all(feature = "onion-service-client", feature = "experimental-api", feature = "keymgr"))] {
-            // TODO: Skipped because running creating multiple TorClient at the same time
-            // is not supported.
-            //
-            // Re-enable these when #1497 is closed.
-            t.skip("tests/testcases/hsc/*.toml");
-            t.skip("tests/testcases/hsc/*.md");
+            t.case("tests/testcases/hsc/*.toml");
+            t.case("tests/testcases/hsc/*.md");
         } else {
             // This is not yet implemented, see #1487
             t.skip("tests/testcases/hsc-feature-missing/*.toml");
