@@ -1,8 +1,8 @@
 #![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
-#![cfg_attr(not(ci_arti_stable), allow(renamed_and_removed_lints))]
-#![cfg_attr(not(ci_arti_nightly), allow(unknown_lints))]
+#![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
+#![allow(unknown_lints)] // @@REMOVE_WHEN(ci_arti_nightly)
 #![warn(missing_docs)]
 #![warn(noop_method_call)]
 #![warn(unreachable_pub)]
@@ -250,7 +250,7 @@ impl<R: Runtime> ChanMgr<R> {
         &self,
         target: &T,
         usage: ChannelUsage,
-    ) -> Result<(Channel, ChanProvenance)> {
+    ) -> Result<(Arc<Channel>, ChanProvenance)> {
         let targetinfo = OwnedChanTarget::from_chan_target(target);
 
         let (chan, provenance) = self.mgr.get_or_launch(targetinfo, usage).await?;
@@ -329,7 +329,7 @@ impl<R: Runtime> ChanMgr<R> {
     pub async fn build_unmanaged_channel(
         &self,
         target: impl tor_linkspec::IntoOwnedChanTarget,
-    ) -> Result<Channel> {
+    ) -> Result<Arc<Channel>> {
         use factory::ChannelFactory as _;
         let target = target.to_owned();
 
