@@ -6,10 +6,10 @@ use crate::{ChanProvenance, ChannelConfig, ChannelUsage, Dormancy, Error, Result
 use crate::factory::BootstrapReporter;
 use async_trait::async_trait;
 use futures::future::{FutureExt, Shared};
+use oneshot_fused_workaround as oneshot;
 use std::result::Result as StdResult;
 use std::sync::Arc;
 use std::time::Duration;
-use tor_async_utils::oneshot;
 use tor_basic_utils::RngExt as _;
 use tor_error::internal;
 use tor_linkspec::{HasRelayIds, RelayIds};
