@@ -43,10 +43,12 @@
 
 pub mod cmdline;
 mod err;
+pub mod file_watcher;
 mod flatten;
 pub mod list_builder;
 pub mod load;
 mod misc;
+pub mod mistrust;
 mod mut_cfg;
 mod path;
 pub mod sources;
