@@ -290,6 +290,18 @@ impl OnionService {
 
         let nickname = config.nickname.clone();
 
+        if config.restricted_discovery.enabled {
+            info!(
+                nickname=%nickname,
+                "Launching onion service in restricted discovery mode"
+            );
+        } else {
+            info!(
+                nickname=%nickname,
+                "Launching onion service"
+            );
+        }
+
         let state_handle = state_dir
             .acquire_instance(&config.nickname)
             .map_err(StartupError::StateDirectoryInaccessible)?;
@@ -407,6 +419,8 @@ impl RunningOnionService {
             })
         })
 
+        // TODO: reconfigure the authorized_clients too
+
         // TODO (#1153, #1209): We need to make sure that the various tasks listening on
         // config_rx actually enforce the configuration, not only on new
         // connections, but existing ones.
