@@ -4,6 +4,7 @@ mod descriptor;
 mod reactor;
 mod reupload_timer;
 
+use crate::config::restricted_discovery::RestrictedDiscoveryKeys;
 use crate::internal_prelude::*;
 
 use backoff::{BackoffError, BackoffSchedule, RetriableError, Runner};
@@ -96,7 +97,7 @@ impl<R: Runtime, M: Mockable> Publisher<R, M> {
             nickname,
             dir_provider,
             mockable,
-            config,
+            &config,
             ipt_watcher,
             config_rx,
             status_tx,
