@@ -14,10 +14,12 @@
 //!    For example, individual cell and message names from `tor-cell`,
 //!    and the types from `tor_proto::stream` other than the high-level `DataStream`.
 
+#![allow(unused_imports)]
+
 pub(crate) use std::{
     cmp::{Ordering, Reverse},
     collections::{BinaryHeap, HashSet},
-    fmt::{self, Debug, Display},
+    fmt::{self, Debug},
     future::Future,
     marker::PhantomData,
     mem,
@@ -30,14 +32,14 @@ pub(crate) use std::{
 pub(crate) use futures::{
     channel::mpsc,
     stream::FusedStream,
-    task::{Spawn, SpawnError, SpawnExt as _},
-    FutureExt as _, Stream, StreamExt as _,
+    task::{noop_waker_ref, Spawn, SpawnError, SpawnExt as _},
+    FutureExt as _, Sink, SinkExt as _, Stream, StreamExt as _,
 };
 
 pub(crate) use {
     derive_builder::Builder,
     derive_deftly::{define_derive_deftly, Deftly},
-    derive_more::{Deref, DerefMut, From, Into},
+    derive_more::{Constructor, Deref, DerefMut},
     educe::Educe,
     pin_project::pin_project,
     serde::{Deserialize, Serialize},
@@ -49,10 +51,12 @@ pub(crate) use {
 };
 
 pub(crate) use {
+    tor_async_utils::stream_peek::StreamUnobtrusivePeeker,
+    tor_basic_utils::ByteQty as Qty,
     tor_config::ConfigBuildError,
     tor_error::{error_report, internal, into_internal, Bug, ErrorKind, HasKind},
     tor_log_ratelim::log_ratelim,
-    tor_rtcompat::CoarseInstant,
+    tor_rtcompat::{CoarseInstant, CoarseTimeProvider},
 };
 
 pub(crate) use crate::{
@@ -60,7 +64,10 @@ pub(crate) use crate::{
     drop_bomb::{DropBomb, DropBombCondition},
     drop_reentrancy,
     error::{Error, ReclaimCrashed, StartupError, TrackerCorrupted},
-    mtracker::Participation,
+    if_enabled::{EnabledToken, IfEnabled},
+    memory_cost::{HasMemoryCost, HasTypedMemoryCost, TypedParticipation},
+    mtracker::{self, Account, IsParticipant, Participation},
+    private::Sealed,
     refcount,
-    utils::{DefaultExtTake, NoopWaker, Qty},
+    utils::DefaultExtTake,
 };
