@@ -18,7 +18,7 @@
 use super::*;
 
 use std::collections::BTreeMap;
-use std::fmt::Write as _;
+use std::fmt::{Display, Write as _};
 use std::time::Duration;
 
 use itertools::Itertools;
@@ -27,19 +27,19 @@ use slotmap::Key as _;
 use tracing_test::traced_test;
 
 use tor_basic_utils::RngExt as _;
-use tor_rtcompat::{CoarseDuration, CoarseTimeProvider as _, Runtime};
+use tor_rtcompat::{CoarseDuration, Runtime};
 use tor_rtmock::MockRuntime;
 
 //---------- useful utilities ----------
 
-pub(crate) const TEST_DEFAULT_LIMIT: usize = mby(20);
-pub(crate) const TEST_DEFAULT_LOWWATER: usize = mby(15);
+pub(crate) const TEST_DEFAULT_LIMIT: usize = mbytes(20);
+pub(crate) const TEST_DEFAULT_LOWWATER: usize = mbytes(15);
 
 fn secs(s: u64) -> CoarseDuration {
     Duration::from_secs(s).into()
 }
 
-pub(crate) const fn mby(mib: usize) -> usize {
+pub(crate) const fn mbytes(mib: usize) -> usize {
     mib * 1024 * 1024
 }
 
@@ -88,6 +88,7 @@ mod consistency {
 
     impl CallerInfoCollector {
         pub(super) fn note_account(&mut self, acct: &Account, reclaimed: ReclaimedOrOk) {
+            let acct = acct.0.as_enabled().unwrap();
             writeln!(self.debug_dump, "acct {acct:?} {reclaimed:?}").unwrap();
             if acct.aid.is_null() || reclaimed.is_err() {
                 return;
@@ -101,6 +102,7 @@ mod consistency {
             reclaimed: ReclaimedOrOk,
             used: usize,
         ) {
+            let p = p.0.as_enabled().unwrap();
             writeln!(self.debug_dump, "particip {p:?} {reclaimed:?} {used:?}").unwrap();
             if p.pid.is_null() || p.aid.is_null() || reclaimed.is_err() {
                 return;
@@ -108,13 +110,14 @@ mod consistency {
             self.note_partn_core(p, used);
         }
         pub(super) fn note_partn_clone(&mut self, p: &Participation) {
+            let p = p.0.as_enabled().unwrap();
             writeln!(self.debug_dump, "partn {p:?}").unwrap();
             if p.pid.is_null() {
                 return;
             }
             self.note_partn_core(p, 0);
         }
-        fn note_partn_core(&mut self, p: &Participation, x_used: usize) {
+        fn note_partn_core(&mut self, p: &ParticipationInner, x_used: usize) {
             let pc = self.pcs.entry((p.aid, *p.pid)).or_default();
             let used = *p.cache.as_raw() + x_used;
             pc.0 += 1;
@@ -127,7 +130,7 @@ mod consistency {
         trk: &Arc<MemoryQuotaTracker>,
         collect_caller_info: impl FnOnce(&mut CallerInfoCollector),
     ) {
-        let state = trk.lock().unwrap();
+        let state = trk.lock().unwrap().into_enabled().unwrap();
 
         let (expected, debug_dump) = {
             let mut c = CallerInfoCollector::default();
@@ -202,10 +205,10 @@ impl TestPartn {
 }
 
 impl IsParticipant for TestPartn {
-    fn get_oldest(&self) -> Option<CoarseInstant> {
+    fn get_oldest(&self, _: EnabledToken) -> Option<CoarseInstant> {
         self.get_oldest()
     }
-    fn reclaim(self: Arc<Self>) -> ReclaimFuture {
+    fn reclaim(self: Arc<Self>, _: EnabledToken) -> ReclaimFuture {
         (*self.clone()).reclaim()
     }
 }
@@ -256,10 +259,10 @@ struct UnifiedP {
 type ReclaimedOrOk = Result<(), ()>;
 
 impl IsParticipant for UnifiedP {
-    fn get_oldest(&self) -> Option<CoarseInstant> {
+    fn get_oldest(&self, _: EnabledToken) -> Option<CoarseInstant> {
         self.state.get_oldest()
     }
-    fn reclaim(self: Arc<Self>) -> ReclaimFuture {
+    fn reclaim(self: Arc<Self>, _: EnabledToken) -> ReclaimFuture {
         self.state.reclaim()
     }
 }
@@ -333,7 +336,7 @@ fn basic() {
             .collect();
 
         for p in &ps[0..19] {
-            p.lock().claim(mby(1)).unwrap();
+            p.lock().claim(mbytes(1)).unwrap();
             UnifiedP::settle_check_consistency(&rt, &trk, &ps).await;
         }
 
@@ -344,10 +347,10 @@ fn basic() {
         for p in &ps[20..] {
             // check that we are exercising a situation with nonzero cached
             // (this is set up by register_participant
-            assert_ne!(p.lock().partn.cache, Qty(0));
+            assert_ne!(p.lock().partn.0.as_enabled().unwrap().cache, Qty(0));
 
             p.lock()
-                .claim(mby(1))
+                .claim(mbytes(1))
                 .expect("allocation rejected, during collapse, but collapse is async");
         }
 
@@ -372,17 +375,17 @@ fn parent() {
             let mk_p = |parent, age, show| UnifiedP::new(&rt, &trk, parent, age, show);
 
             let parent = mk_p(None, parent_age, "parent");
-            parent.lock().claim(mby(7)).unwrap();
+            parent.lock().claim(mbytes(7)).unwrap();
             rt.advance_until_stalled().await;
             assert!(parent.is_reclaimed().is_ok());
 
             let child = mk_p(Some(&parent.acct), child_age, "child");
-            child.lock().claim(mby(7)).unwrap();
+            child.lock().claim(mbytes(7)).unwrap();
             assert!(parent.is_reclaimed().is_ok());
             assert!(child.is_reclaimed().is_ok());
 
             let trigger = mk_p(None, secs(0), "trigger");
-            trigger.lock().claim(mby(7)).unwrap();
+            trigger.lock().claim(mbytes(7)).unwrap();
             assert!(trigger.is_reclaimed().is_ok());
 
             rt.advance_until_stalled().await;
@@ -474,10 +477,10 @@ fn explicit_destroy() {
         let p0 = UnifiedP::new(&rt, &trk, None, secs(0), "0");
         let p1 = p0.clone();
 
-        p0.lock().claim(mby(1)).unwrap();
+        p0.lock().claim(mbytes(1)).unwrap();
         UnifiedP::settle_check_consistency(&rt, &trk, [&p0]).await;
 
-        p1.lock().claim(mby(2)).unwrap();
+        p1.lock().claim(mbytes(2)).unwrap();
         UnifiedP::settle_check_consistency(&rt, &trk, [&p0]).await;
 
         p1.lock().partn.clone().destroy_participant();
@@ -488,7 +491,7 @@ fn explicit_destroy() {
             // We don't note the participation, since it's dead.
         });
 
-        assert!(p1.lock().claim(mby(3)).is_err());
+        assert!(p1.lock().claim(mbytes(3)).is_err());
 
         // Now we drop everything.  This exercises much of the teardown!
     });
@@ -576,8 +579,8 @@ fn complex() {
             });
         };
 
-        up.lock().claim(mby(1)).unwrap();
-        ah.ps[0].lock().claim(mby(11)).unwrap();
+        up.lock().claim(mbytes(1)).unwrap();
+        ah.ps[0].lock().claim(mbytes(11)).unwrap();
 
         settle_check_consistency().await;
 
@@ -586,7 +589,7 @@ fn complex() {
             assert!(p.is_reclaimed().is_ok());
         }
 
-        ah.ps[1].lock().claim(mby(11)).unwrap();
+        ah.ps[1].lock().claim(mbytes(11)).unwrap();
 
         settle_check_consistency().await;
         assert!(up.is_reclaimed().is_ok());
@@ -602,10 +605,10 @@ fn complex() {
 struct DummyParticipant;
 
 impl IsParticipant for DummyParticipant {
-    fn get_oldest(&self) -> Option<CoarseInstant> {
+    fn get_oldest(&self, _: EnabledToken) -> Option<CoarseInstant> {
         None
     }
-    fn reclaim(self: Arc<Self>) -> ReclaimFuture {
+    fn reclaim(self: Arc<Self>, _: EnabledToken) -> ReclaimFuture {
         Box::pin(async { Reclaimed::Collapsing })
     }
 }
@@ -664,7 +667,7 @@ fn errors() {
             let p = ah.ps.pop().unwrap();
             let mut state = Arc::into_inner(p).unwrap().state.into_inner().unwrap();
 
-            state.claim(mby(30)).unwrap(); // will trigger reclaim, which discovers the loss
+            state.claim(mbytes(30)).unwrap(); // will trigger reclaim, which discovers the loss
 
             rt.advance_until_stalled().await;
             check_consistency_general(&trk, |collector| {
@@ -678,7 +681,7 @@ fn errors() {
         // Reclaimed account
         {
             let ah = mk_ah();
-            ah.ps[0].lock().claim(mby(30)).unwrap();
+            ah.ps[0].lock().claim(mbytes(30)).unwrap();
 
             rt.advance_until_stalled().await;
             check_consistency_general(&trk, |_collector| ());
@@ -689,14 +692,14 @@ fn errors() {
             assert_error!(AccountClosed, p.lock().claim(CLAIM));
 
             let cloned = ah.acct.clone();
-            assert!(cloned.aid.is_null());
+            assert!(cloned.0.as_enabled().unwrap().aid.is_null());
             assert_error!(
                 AccountClosed,
                 ah.acct.register_participant(dummy_dangling())
             );
 
             let mut cloned = p.lock().partn.clone();
-            assert!(cloned.pid.is_null());
+            assert!(cloned.0.as_enabled().unwrap().pid.is_null());
             assert_error!(AccountClosed, cloned.claim(CLAIM));
 
             // but we can still release
