@@ -19,7 +19,7 @@
 pub(super) mod syncview;
 
 use super::handshake::RelayCryptLayerProtocol;
-use super::streammap::{EndSentStreamEnt, OpenStreamEnt, ShouldSendEnd, StreamEntMut};
+use super::streammap::{EndSentStreamEnt, ShouldSendEnd, StreamEntMut};
 use super::MutableState;
 use crate::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
 use crate::circuit::handshake::{BoxedClientLayer, HandshakeRole};
@@ -41,7 +41,6 @@ use crate::util::sometimes_unbounded_sink::SometimesUnboundedSink;
 use crate::util::SinkExt as _;
 use crate::{Error, Result};
 use std::borrow::Borrow;
-use std::collections::VecDeque;
 use std::marker::PhantomData;
 use std::pin::Pin;
 use tor_cell::chancell::msg::{AnyChanMsg, HandshakeType, Relay};
@@ -50,6 +49,7 @@ use tor_cell::relaycell::{
     AnyRelayMsgOuter, RelayCellDecoder, RelayCellFormat, RelayCellFormatTrait, RelayCellFormatV0,
     RelayCmd, StreamId, UnparsedRelayMsg,
 };
+use tor_error::internal;
 #[cfg(feature = "hs-service")]
 use {
     crate::stream::{DataCmdChecker, IncomingStreamRequest, IncomingStreamRequestFilter},
@@ -59,8 +59,7 @@ use {
 use futures::channel::mpsc;
 use futures::Stream;
 use futures::{Sink, StreamExt};
-use tor_async_utils::oneshot;
-use tor_error::internal;
+use oneshot_fused_workaround as oneshot;
 
 use std::sync::{Arc, Mutex};
 use std::task::{Context, Poll};
@@ -329,18 +328,6 @@ pub(super) struct CircHop {
     recvwindow: sendme::CircRecvWindow,
     /// Window used to say how many cells we can send.
     sendwindow: sendme::CircSendWindow,
-    /// Buffer for messages we can't send to this hop yet due to congestion control.
-    ///
-    /// Contains the cell to send, and a boolean equivalent to the `early` parameter
-    /// in `Reactor::send_relay_cell` (as in, whether to send the cell using `RELAY_EARLY`).
-    ///
-    /// This shouldn't grow unboundedly: we try and pop things off it first before
-    /// doing things that would result in it growing (and stop before growing it
-    /// if popping things off it can't be done).
-    ///
-    /// NOTE: Control messages could potentially add unboundedly to this, although that's
-    ///       not likely to happen (and isn't triggereable from the network, either).
-    outbound: VecDeque<(bool, AnyRelayMsgOuter)>,
     /// Decodes relay cells received from this hop.
     inbound: RelayCellDecoder,
 }
@@ -361,7 +348,6 @@ impl CircHop {
             map: streammap::StreamMap::new(),
             recvwindow: sendme::CircRecvWindow::new(1000),
             sendwindow: sendme::CircSendWindow::new(initial_window),
-            outbound: VecDeque::new(),
             inbound: RelayCellDecoder::new(format),
         }
     }
@@ -891,131 +877,70 @@ impl Reactor {
                 }
             }
 
-            // Now for the tricky part. We want to grab some relay cells from all of our streams
-            // and forward them on to the channel, but we need to pay attention to both whether
-            // the channel can accept cells right now, and whether congestion control allows us
-            // to send them.
-            //
-            // We also have to do somewhat cursed things and call start_send inside this poll_fn,
-            // since we need to check whether the channel can still receive cells after each one
-            // that we send.
-
-            let mut streams_to_close = vec![];
-            let mut stream_relaycells = vec![];
-
-            // (using this as a named block for early returns; not actually a loop)
-            #[allow(clippy::never_loop)]
-            'outer: loop {
-                // First, drain our queue of things we tried to send earlier, but couldn't.
-                //
-                // (It's not clear to me that this is needed.  We call this as the first thing
-                // for each hop, below.  So this would only be relevant if there were no hops.
-                // TODO #1397 the manual handling of Pending as `break 'outer` is weird.)
+            // Check each hop for an outbound message pending.
+            for i in 0..self.hops.len() {
                 if !self.chan_sender.poll_ready_unpin_bool(cx)? {
-                    break 'outer;
+                    // Channel isn't ready to send; we can't act on anything else.
+                    // (Even processing an end-of-stream would end up having to buffer
+                    // an END message in the channel).
+                    break;
                 }
-
-                // Let's look at our hops, and streams for each hop.
-                for i in 0..self.hops.len() {
-                    let hop_num = HopNum::from(i as u8);
-                    // If we can, drain our queue of things we tried to send earlier, but
-                    // couldn't due to congestion control.
-                    'hop_outbound: loop {
-                        if !self.chan_sender.poll_ready_unpin_bool(cx)? {
-                            break 'outer;
-                        }
-                        if self.hops[i].sendwindow.window() == 0 {
-                            break 'hop_outbound;
-                        }
-                        let Some((early, cell)) = self.hops[i].outbound.pop_front() else {
-                            break 'hop_outbound;
-                        };
-                        trace!(
-                            "{}: sending from hop-{}-enqueued: {:?}",
-                            self.unique_id,
-                            i,
-                            cell
-                        );
-                        // We know the stream has a non-empty SENDME
-                        // window because we accept at most one cell per
-                        // stream below, and only when the relevant
-                        // stream has a non-empty window and after
-                        // clearing the backlog here.
-                        //
-                        // TODO prop340: We need to be careful here when
-                        // adding fragmentation; e.g. this argument
-                        // fails to hold if we allow later breaking a
-                        // DATA message into multiple messages and cells
-                        // for packing.
-                        self.send_relay_cell(cx, hop_num, early, cell)?;
-                    }
-                    let hop = &mut self.hops[i];
-                    // Look at all of the streams on this hop.
-                    for (id, stream) in hop.map.iter_mut() {
-                        let StreamEntMut::Open(OpenStreamEnt {
-                            rx, send_window, ..
-                        }) = stream
-                        else {
-                            continue;
-                        };
-                        // Do the stream and hop send windows allow us to obtain and
-                        // send something?
-                        //
-                        // NOTE: not everything counts toward congestion
-                        // control. However, we can't easily remove
-                        // this check:
-                        // * We need to be careful not to buffer more
-                        // than ONE message per stream for the call to
-                        // `send_relay_cell` above, and potentially
-                        // closing the stream below, to be correct.
-                        // * We need to be careful about allowing
-                        // messages that *don't* count to be sent before
-                        // messages that *do*; e.g. we wouldn't want to
-                        // accept and send an END message on a stream where we
-                        // still have DATA messages queued.
-                        if send_window.window() > 0 && hop.sendwindow.window() > 0 {
-                            match Pin::new(rx).poll_next(cx) {
-                                Poll::Ready(Some(m)) => {
-                                    stream_relaycells
-                                        .push((hop_num, AnyRelayMsgOuter::new(Some(id), m)));
-                                }
-                                Poll::Ready(None) => {
-                                    // Stream channel was closed, so we'll close the stream.
-                                    //
-                                    // This can happen either when the last StreamTarget for the
-                                    // stream is dropped, or when StreamTarget::close() is called
-                                    // on one of the stream targets.
-                                    //
-                                    // We know there are no queued messages for the stream
-                                    // since we already flushed those above.
-                                    //
-                                    // We can't close it here due to
-                                    // borrowck; that will happen later.
-                                    streams_to_close.push((hop_num, id));
-                                }
-                                Poll::Pending => {}
-                            }
-                        }
-                    }
+                if self.hops[i].sendwindow.window() == 0 {
+                    // We can't send anything on this hop that counts towards SENDME windows.
+                    //
+                    // In theory we could send messages that don't count towards
+                    // windows (like `RESOLVE`), and process end-of-stream
+                    // events (to send an `END`), but it's probably not worth
+                    // doing an O(N) iteration over flow-control-ready streams
+                    // to see if that's the case.
+                    //
+                    // This *doesn't* block outgoing flow-control messages (e.g.
+                    // SENDME), which are initiated via the control-message
+                    // channel, handled above.
+                    //
+                    // TODO: Consider revisiting. OTOH some extra throttling when circuit-level
+                    // congestion control has "bottomed out" might not be so bad, and the
+                    // alternatives have complexity and/or performance costs.
+                    continue;
                 }
-
-                break;
-            }
-
-            // Close the streams we said we'd close.
-            for (hopn, id) in streams_to_close {
-                self.close_stream(
-                    cx,
-                    hopn,
-                    id,
-                    CloseStreamBehavior::default(),
-                    streammap::TerminateReason::StreamTargetClosed,
-                )?;
-                did_things = true;
-            }
-            // Send messages we said we'd send.
-            for (hopn, rc) in stream_relaycells {
-                self.send_relay_cell(cx, hopn, false, rc)?;
+                let hop_num = HopNum::from(i as u8);
+                // Process an outbound message from the first ready stream on
+                // this hop. The stream map implements round robin scheduling to
+                // ensure fairness across streams.
+                // TODO: Consider looping here to process multiple ready
+                // streams. Need to be careful though to balance that with
+                // continuing to service incoming and control messages.
+                let Some((sid, msg)) = self.hops[i].map.poll_ready_streams_iter(cx).next() else {
+                    // No ready streams for this hop.
+                    continue;
+                };
+                if msg.is_none() {
+                    // Sender was dropped, so close the stream, which
+                    // also removes this entry from the streams iterator.
+                    self.close_stream(
+                        cx,
+                        hop_num,
+                        sid,
+                        CloseStreamBehavior::default(),
+                        streammap::TerminateReason::StreamTargetClosed,
+                    )?;
+                    did_things = true;
+                    continue;
+                };
+                let msg = self.hops[i]
+                    .map
+                    .take_ready_msg(sid)
+                    .expect("msg disappeared");
+                debug_assert!(
+                    {
+                        let Some(StreamEntMut::Open(s)) = self.hops[i].map.get_mut(sid) else {
+                            panic!("Stream {sid} disappeared");
+                        };
+                        s.can_send(&msg)
+                    },
+                    "Stream {sid} produced a message it can't send: {msg:?}"
+                );
+                self.send_relay_cell(cx, hop_num, false, AnyRelayMsgOuter::new(Some(sid), msg))?;
                 did_things = true;
             }
 
@@ -1480,11 +1405,8 @@ impl Reactor {
 
     /// Encode `msg`, encrypt it, and send it to the 'hop'th hop.
     ///
-    /// If there is insufficient outgoing *circuit-level* SENDME window, the
-    /// `msg` is enqueued in the hop's `outbound` queue instead.
-    ///
-    /// If there is insufficient outgoing *stream-level* SENDME window,
-    /// an error is returned instead.
+    /// If there is insufficient outgoing *circuit-level* or *stream-level*
+    /// SENDME window, an error is returned instead.
     ///
     /// Does not check whether the cell is well-formed or reasonable.
     fn send_relay_cell(
@@ -1496,24 +1418,22 @@ impl Reactor {
     ) -> Result<()> {
         let c_t_w = sendme::cmd_counts_towards_windows(msg.cmd());
         let stream_id = msg.stream_id();
-        // Check whether the hop send window is empty, if this cell counts towards windows.
-        // NOTE(eta): It is imperative this happens *before* calling encrypt() below, otherwise
-        //            we'll have cells rejected due to a protocol violation! (Cells have to be
-        //            sent out in the order they were passed to encrypt().)
+        let hop_num = Into::<usize>::into(hop);
+        let circhop = &mut self.hops[hop_num];
+        // We need to apply stream-level flow control *before* encoding the message.
         if c_t_w {
-            let hop_num = Into::<usize>::into(hop);
-            let circhop = &mut self.hops[hop_num];
-            if circhop.sendwindow.window() == 0 {
-                // Send window is empty! Push this cell onto the hop's outbound queue, and it'll
-                // get sent later.
-                trace!(
-                    "{}: having to use onto hop {} queue for cell: {:?}",
-                    self.unique_id,
-                    hop.display(),
-                    msg
-                );
-                circhop.outbound.push_back((early, msg));
-                return Ok(());
+            if let Some(stream_id) = stream_id {
+                let Some(StreamEntMut::Open(ent)) = circhop.map.get_mut(stream_id) else {
+                    warn!(
+                        "{}: sending a relay cell for non-existent or non-open stream with ID {}!",
+                        self.unique_id, stream_id
+                    );
+                    return Err(Error::CircProto(format!(
+                        "tried to send a relay cell on non-open stream {}",
+                        sv(stream_id),
+                    )));
+                };
+                ent.take_capacity_to_send(msg.msg())?;
             }
         }
         let mut body: RelayCellBody = msg
@@ -1532,31 +1452,7 @@ impl Reactor {
         // If the cell counted towards our sendme window, decrement
         // that window, and maybe remember the authentication tag.
         if c_t_w {
-            let hop_num = Into::<usize>::into(hop);
-            let hop = &mut self.hops[hop_num];
-            // checked by earlier conditional, so this shouldn't fail
-            hop.sendwindow.take(tag)?;
-            if let Some(stream_id) = stream_id {
-                // We need to decrement the stream-level sendme window.
-                // Stream data cells should only be dequeued and fed into this function if
-                // the window is above zero, so we don't need to worry about
-                // enqueuing things.
-                match hop.map.get_mut(stream_id) {
-                    Some(StreamEntMut::Open(OpenStreamEnt { send_window, .. })) => {
-                        send_window.take(&())?;
-                    }
-                    _ => {
-                        warn!(
-                            "{}: sending a relay cell for non-existent or non-open stream with ID {}!",
-                            self.unique_id, stream_id
-                        );
-                        return Err(Error::CircProto(format!(
-                            "tried to send a relay cell on non-open stream {}",
-                            sv(stream_id),
-                        )));
-                    }
-                }
-            }
+            circhop.sendwindow.take(tag)?;
         }
         self.send_msg_direct(cx, msg)
     }
@@ -1949,7 +1845,7 @@ impl Reactor {
                         );
                     }
                     if let Some(incomplete) = incomplete {
-                        debug!("{id}: Ignoring parial relay msg received after triggering shutdown: {incomplete:?}", id=self.unique_id);
+                        debug!("{id}: Ignoring partial relay msg received after triggering shutdown: {incomplete:?}", id=self.unique_id);
                     }
                     return Ok(CellStatus::CleanShutdown);
                 }
@@ -1988,13 +1884,7 @@ impl Reactor {
             .hop_mut(hopnum)
             .ok_or_else(|| Error::CircProto("Cell from nonexistent hop!".into()))?;
         match hop.map.get_mut(streamid) {
-            Some(StreamEntMut::Open(OpenStreamEnt {
-                sink,
-                send_window,
-                dropped,
-                cmd_checker,
-                ..
-            })) => {
+            Some(StreamEntMut::Open(ent)) => {
                 // The stream for this message exists, and is open.
 
                 if msg.cmd() == RelayCmd::SENDME {
@@ -2005,13 +1895,14 @@ impl Reactor {
                     // We need to handle sendmes here, not in the stream's
                     // recv() method, or else we'd never notice them if the
                     // stream isn't reading.
-                    send_window.put(Some(()))?;
+                    ent.put_for_incoming_sendme()?;
                     return Ok(CellStatus::Continue);
                 }
 
-                let message_closes_stream = cmd_checker.check_msg(&msg)? == StreamStatus::Closed;
+                let message_closes_stream =
+                    ent.cmd_checker.check_msg(&msg)? == StreamStatus::Closed;
 
-                if let Err(e) = sink.try_send(msg) {
+                if let Err(e) = ent.sink.try_send(msg) {
                     if e.is_full() {
                         // If we get here, we either have a logic bug (!), or an attacker
                         // is sending us more cells than we asked for via congestion control.
@@ -2025,7 +1916,7 @@ impl Reactor {
                         // that we received a cell that we couldn't queue for it.
                         //
                         // Later this value will be recorded in a half-stream.
-                        *dropped += 1;
+                        ent.dropped += 1;
                     }
                 }
                 if message_closes_stream {
